/***************************************************************************
 *   Copyright (C) 2010 by Rico Rommel   *
 *   rico@bierrommel.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "omgps.h"
#include <QApplication>
#include <QDebug>
#include <QDBusReply>

OMGPS::OMGPS(QWidget *parent)
 : QWidget(parent)
{

	IconBasePath = "/usr/share/pixmaps/openmoko-panel-plugin-qt/";
	activeIconFile = "gps_1.png";
	inactiveIconFile = "gps_0.png";


	activeIcon = QIcon(IconBasePath + activeIconFile);
	inactiveIcon = QIcon(IconBasePath + inactiveIconFile);

	MainLayout = new QVBoxLayout();
	ActivateLayout = new QHBoxLayout();

	
	StatusPanelName = QObject::tr("status:");
	FixPanelName = QObject::tr("fix:");
	LatPanelName = QObject::tr("latitude:");
	LongPanelName = QObject::tr("longitude:");
	AltPanelName = QObject::tr("altitude:");

	FixString << QObject::tr("Invalid") << QObject::tr("none") << QObject::tr("2D") << QObject::tr("3D");


	DeviceBox = new OMInfoBox(QObject::tr("Device"), this);
	DeviceBox->addPanel(StatusPanelName,"");
	DeviceBox->addPanel(FixPanelName,"");
	MainLayout->addWidget( DeviceBox );

	PositionBox = new OMInfoBox(QObject::tr("Position"), this);
	PositionBox->addPanel(LatPanelName,"");
	PositionBox->addPanel(LongPanelName,"");
	PositionBox->addPanel(AltPanelName,"");
	MainLayout->addWidget( PositionBox );


	ActivateButton = new QPushButton(QObject::tr("turn GPS on"));
	ActivateLayout->addWidget(ActivateButton);
	connect(ActivateButton,SIGNAL(clicked()),this,SLOT(ActivateButtonclicked()));

	DeactivateButton = new QPushButton(QObject::tr("turn GPS off"));
	ActivateLayout->addWidget(DeactivateButton);
	connect(DeactivateButton,SIGNAL(clicked()),this,SLOT(DeactivateButtonclicked()));

	MainLayout->addLayout(ActivateLayout);


	DialogButtons = new OMDialogButtons(OMFlags::QuitButton | OMFlags::CloseButton | OMFlags::ConfigButton, this);
	MainLayout->addWidget( DialogButtons );
	connect(DialogButtons, SIGNAL(clicked(QFlags< OMFlags :: Flag >)), this, SLOT(DialogButtonClicked(QFlags< OMFlags :: Flag >)));

	setLayout( MainLayout );


	Configuration = new OMConfig("openmoko-panel-plugin-qt-gps", 0 );
	

	SysTrayIcon = new QSystemTrayIcon( inactiveIcon, this );
	connect(SysTrayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));
	SysTrayIcon->show();

	systemBus = new QDBusConnection( QDBusConnection::connectToBus( QDBusConnection::SystemBus, "system" ) );
	connectToUsageInterface();
	connectToGypsyInterface();
	connectToGypsyPositionInterface();


	if (systemBus->connect("org.freesmartphone.ousaged", "/org/freesmartphone/Usage", "org.freesmartphone.Usage","ResourceChanged", this, SLOT(ResourceChanged(QString,bool)) )) qDebug() << "Signal ResourceChanged connected";


}


OMGPS::~OMGPS()
{
}




void OMGPS::iconActivated(QSystemTrayIcon::ActivationReason)
{
    showMaximized ();
}



void OMGPS::DeactivateButtonclicked()
{
	Configuration->runPreDisableScript();
	if (UsageInterface->isValid()) UsageInterface->call("ReleaseResource","GPS");
}


void OMGPS::ActivateButtonclicked()
{
	Configuration->runPreEnableScript();
    	if (UsageInterface->isValid()) UsageInterface->call("RequestResource","GPS");

}


void OMGPS::connectToUsageInterface()
{
    	UsageInterface = new OMUsageInterface( this);
	
	if (UsageInterface->isValid()) {
		
		QDBusReply <bool> reply = UsageInterface->call("GetResourceState","GPS");
		ResourceChanged("GPS", reply.value());
		
		}
	else qDebug() << "could not connect to usage interface";
}

void OMGPS::ResourceChanged(QString name, bool state)
{
    if (name == "GPS") {

	if (state) {
		SysTrayIcon->setIcon( activeIcon );	
		DeviceBox->changePanelValue( StatusPanelName, QObject::tr("active") );
		Configuration->runPostEnableScript();
		}

	else {

		SysTrayIcon->setIcon( inactiveIcon );		
		DeviceBox->changePanelValue( StatusPanelName, QObject::tr("inactive") );
		Configuration->runPostDisableScript();


		}

	}

}


void OMGPS::connectToGypsyInterface()
{
    	GypsyInterface = new QDBusInterface("org.freedesktop.Gypsy", "/org/freedesktop/Gypsy", "org.freedesktop.Gypsy.Device", *systemBus, this);

	if (GypsyInterface->isValid()) {
		
		connect(GypsyInterface, SIGNAL(FixStatusChanged(int)),this,SLOT(FixStatusChanged(int)));
		
		}
	else qDebug() << "could not connect to gypsy interface";
}


void OMGPS::FixStatusChanged(int i)
{

	DeviceBox->changePanelValue( FixPanelName,  FixString.at( i )  );
}

void OMGPS::connectToGypsyPositionInterface()
{
    	GypsyPositionInterface = new QDBusInterface("org.freedesktop.Gypsy", "/org/freedesktop/Gypsy", "org.freedesktop.Gypsy.Position", *systemBus, this);

	if (GypsyPositionInterface->isValid()) {
		
		connect(GypsyPositionInterface, SIGNAL(PositionChanged(int,int,double,double,double)),this,SLOT(PositionChanged(int,int,double,double,double)));
		
		}
	else qDebug() << "could not connect to gypsy position interface";
}

void OMGPS::PositionChanged(int field, int tstamp ,double lat ,double lgt, double alt)
{
    PositionBox->changePanelValue( LatPanelName, QString::number( lat ) );
    PositionBox->changePanelValue( LongPanelName, QString::number( lgt ) );
    PositionBox->changePanelValue( AltPanelName, QString::number( alt ) );
}


void OMGPS::DialogButtonClicked(QFlags<OMFlags::Flag> flags)
{
    if ( flags.testFlag(OMFlags::QuitButton) ) qApp->quit();
    if ( flags.testFlag(OMFlags::CloseButton) ) hide();
    if ( flags.testFlag(OMFlags::ConfigButton) ) Configuration->showMaximized();

}
